/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* $Id: xmlconfig.c,v 1.1 2002/10/19 20:44:59 dolphy Exp $ */
 
#include <xmlconfig.h>

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <string.h>

/* ************************************************************************** */

typedef struct {

	/* Public properties */
	xmlChar		* fname;
	
	/* libxml stuff */
	xmlDocPtr				xml_doc_ptr;
	xmlXPathContextPtr		xpath_context_ptr;

} _xmlconfig;

/* ************************************************************************** */

xmlconfig *
xmlconfig_open (const char * fname)
{

	_xmlconfig	* _x = NULL;

	/* Create the structure */
	_x = (_xmlconfig *) malloc (sizeof (_xmlconfig));
	if (! _x) return NULL;
	memset (_x, 0, sizeof (_xmlconfig));

	/* Copy the name */
	_x -> fname = xmlCharStrdup (fname);
	if (! _x -> fname) {
		xmlconfig_close ((xmlconfig *) _x);
		return NULL; }

	/* Do not want to parse spaces !! */
	xmlKeepBlanksDefault (0);

	/* Open the xml document */
	_x -> xml_doc_ptr = xmlParseFile (_x -> fname);
	if (! _x -> xml_doc_ptr) {
		xmlconfig_close ((xmlconfig *) _x);
		return NULL; }
	
	/* Initiate the XPath environment */
	xmlXPathInit ();

	/* Create a new xmlXPathContext */
	_x -> xpath_context_ptr = xmlXPathNewContext (_x -> xml_doc_ptr);
	if (! _x -> xpath_context_ptr) {
		xmlconfig_close ((xmlconfig *) _x);
		return NULL; }

	/* Done, return */
	return ((xmlconfig *) _x);
}

/* ************************************************************************** */

int
xmlconfig_save (xmlconfig * xconfig, const char * fname)
{

	char		* output;
	int			res;
	_xmlconfig	* _x = (_xmlconfig *) xconfig;
	
	/* Check parameters */
	if (! _x) return -1;
	if (fname) output = (char *) fname;
	else output = (char *) _x -> fname;
	
	/* And save to file */
	res = xmlSaveFormatFile (output, _x -> xml_doc_ptr, 1);
	xmlCleanupParser ();
	if (res == -1) return -1;
	else return 0;
}

/* ************************************************************************** */

int
xmlconfig_dump (xmlconfig * xconfig, FILE * stream)
{

	int			res;
	FILE		* output;
	_xmlconfig	* _x = (_xmlconfig *) xconfig;
	
	/* Check parameters */
	if (! _x) return -1;
	if (! stream) output = stdout;
	else output = stream;
	
	/* And dump through a file pointer */
	res = xmlDocDump (stream, _x -> xml_doc_ptr);
	xmlCleanupParser ();
	if (res == -1) return -1;
	else return 0;
}

/* ************************************************************************** */

/* Frees all resources used for the xml configuration file */

void
xmlconfig_close (xmlconfig * xconfig)
{

	_xmlconfig	* _x = (_xmlconfig *) xconfig;

	/* Check parameters */
	if (! _x) return;

	/* Check if there is an active context */
	if (_x -> xpath_context_ptr) {
		xmlXPathFreeContext (_x -> xpath_context_ptr);
		_x -> xpath_context_ptr = NULL; }

	/* Close the document */
	if (_x -> xml_doc_ptr) {
		xmlFreeDoc (_x -> xml_doc_ptr);
		_x -> xml_doc_ptr = NULL; }

	/* Cleanup any parser */
	xmlCleanupParser ();

	/* Free the fname */
	if (_x -> fname) {
		free (_x -> fname);
		_x -> fname = NULL; }

	/* Free the structure and return */
	free (_x);
	return;
}

/* ************************************************************************** */

/* Get a copy of the node content - See for text */

char *
xmlconfig_get_content_pos (
							xmlconfig * xconfig,
							const char * path, unsigned int pos)
{

	_xmlconfig			* _x = (_xmlconfig *) xconfig;
	xmlXPathObjectPtr	xpath_object_ptr = NULL;
	xmlNodePtr			node_ptr;
	char				* value;

	/* Parameters check */
	if (! _x || ! _x -> xpath_context_ptr) return NULL;
	if (! path) return NULL;

	/* Go evaluate the expression */
	xpath_object_ptr = xmlXPathEvalExpression (path, _x -> xpath_context_ptr);
	if (! xpath_object_ptr) return NULL;
	
	/* Now get the node pointer */
	if (! xpath_object_ptr -> nodesetval) {
		xmlXPathFreeObject (xpath_object_ptr);
		return NULL; }
	if (! xpath_object_ptr -> nodesetval -> nodeTab) {
		xmlXPathFreeObject (xpath_object_ptr);
		return NULL; }
	
	if (pos >= xpath_object_ptr -> nodesetval -> nodeNr) {
		xmlXPathFreeObject (xpath_object_ptr);
		return NULL; }

	node_ptr = xpath_object_ptr -> nodesetval -> nodeTab [pos];
	if (! node_ptr) {
		xmlXPathFreeObject (xpath_object_ptr);
		return NULL; }

	/* Now get the value */
	value = xmlNodeGetContent (node_ptr);
	if (value && ! * value) { free (value); value = NULL; }

	/* Free resources */
	xmlXPathFreeObject (xpath_object_ptr);
	xmlCleanupParser ();

	/* And return */
	return (value);
}

/* ************************************************************************** */

char *
xmlconfig_get_attribute_pos (
								xmlconfig * xconfig,
								char * path, char * attrname,
								unsigned int pos)
{

	_xmlconfig			* _x = (_xmlconfig *) xconfig;
	xmlXPathObjectPtr	xpath_object_ptr = NULL;
	xmlNodePtr			node_ptr;
	char				* value;

	/* Parameters check */
	if (! _x || ! _x -> xpath_context_ptr) return NULL;
	if (! path) return NULL;
	if (! attrname) return NULL;

	/* Go on getting the node */
	xpath_object_ptr = xmlXPathEval (path, _x -> xpath_context_ptr);
	if (! xpath_object_ptr) return NULL;

	/* Now get the node pointer */
	if (! xpath_object_ptr -> nodesetval) {
		xmlXPathFreeObject (xpath_object_ptr);
		return NULL; }
	if (! xpath_object_ptr -> nodesetval -> nodeTab) {
		xmlXPathFreeObject (xpath_object_ptr);
		return NULL; }
	
	if (pos >= xpath_object_ptr -> nodesetval -> nodeNr) {
		xmlXPathFreeObject (xpath_object_ptr);
		return NULL; }

	node_ptr = xpath_object_ptr -> nodesetval -> nodeTab [pos];
	if (! node_ptr) {
		xmlXPathFreeObject (xpath_object_ptr);
		return NULL; }

	/* Get the attribute */
	value = xmlGetProp (node_ptr, attrname);
	if (value && ! * value) { free (value); value = NULL; }

	/* Free resources */
	xmlXPathFreeObject (xpath_object_ptr);
	xmlCleanupParser ();
	
	/* And return */
	return (value);
}

/* ************************************************************************** */

int
xmlconfig_set_content_pos (
							xmlconfig * xconfig,
							char * path, char * value,
							unsigned int pos)
{

	_xmlconfig			* _x = (_xmlconfig *) xconfig;
	xmlXPathObjectPtr	xpath_object_ptr = NULL;
	xmlNodePtr			node_ptr;
	
	/* Parameters check */
	if (! _x || ! _x -> xpath_context_ptr) return -1;
	if (! path) return -1;
	if (! value) return -1;

	/* Go on getting the node */
	xpath_object_ptr = xmlXPathEval (path, _x -> xpath_context_ptr);
	if (! xpath_object_ptr) return -1;

	/* Now get the node pointer */
	if (! xpath_object_ptr -> nodesetval) {
		xmlXPathFreeObject (xpath_object_ptr);
		return -1; }
	if (! xpath_object_ptr -> nodesetval -> nodeTab) {
		xmlXPathFreeObject (xpath_object_ptr);
		return -1; }
	
	if (pos >= xpath_object_ptr -> nodesetval -> nodeNr) {
		xmlXPathFreeObject (xpath_object_ptr);
		return -1; }

	node_ptr = xpath_object_ptr -> nodesetval -> nodeTab [pos];
	if (! node_ptr) {
		xmlXPathFreeObject (xpath_object_ptr);
		return -1; }

	/* Now set the value */
	xmlNodeSetContent (node_ptr, value);
			
	/* Free resources */
	xmlXPathFreeObject (xpath_object_ptr);

	/* And return */
	return 0;
}

/* ************************************************************************** */
